#----------------------------------------------------------------------------------------------------#
# Filename: main_text_analyses.R
# File purpose: Conduct analyses reported in text of Engelhardt "Trumped by Race: Explanations for Race's Influence on Whites' Votes in 2016"
#----------------------------------------------------------------------------------------------------#

# setwd()

#---Install Packages----
source("00_package_install.R")

#---Load Packages----
library(car) 
library(stargazer)
library(gridExtra)
library(ggplot2)
library(ggthemes)
library(lavaan)
library(semTools)
library(xtable)
library(plyr)

#---Loading cleaning files----
source("01_anes_cumulative_cleaning.R")
source("01_anes_2016_cleaning.R")
source("01_vsg_cleaning.R")

#---Figure 1----
# adding variables 
anes16$weight <- anes16$weight.full
anes16$year <- 2016

# stacking data sets for analyses
cdat <- rbind(anes[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                      "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")], 
              anes16[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                        "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")])

cdat <- subset(cdat, year >= 1986)


YEAR <- unique(cdat$year[which(!is.na(cdat$rr_sc) & !is.na(cdat$rep_2pvote))])
for(i in 1:length(YEAR)){
  m <- lm(rep_2pvote ~ pid7_sc + ideo_sc + rr_sc 
          + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == YEAR[i], 
          weights = weight)
  COEF <- coef(m)[-1]
  SE <- sqrt(diag(vcov(m)))[-1]
  x <- cbind(COEF, SE, rep(YEAR[i], length(COEF)))
  assign(paste("OUT_full", YEAR[i], sep = "."), x)
}

DFs <- grep("OUT_full.", ls(), value = T)
OUT.full.year <- data.frame(coef = NA, se = NA, year = NA)
for(i in DFs){
  x <- get(i)
  colnames(x) <- c("coef", "se", "year")
  OUT.full.year <- rbind(OUT.full.year, x)
}
OUT.full.year <- OUT.full.year[-1,]
OUT.full.year$var <- rep(rownames(OUT_full.1988), length(DFs))
rownames(OUT.full.year) <- NULL

OUT.full.year$var <- factor(OUT.full.year$var, levels = c("rr_sc", "pid7_sc", "ideo_sc",
                                                          "age_sc", "col", "fem", "inc_sc", "south"),
                            labels = c("Racial Resentment", "Partisanship", "Ideology",
                                       "Age", "College Degree", "Female", "Income", "Southerner"))

outsub <- subset(OUT.full.year, var == "Racial Resentment")

# Plotting
p1 <- ggplot(outsub, 
             aes(x = year, y = coef)) +
  geom_segment(aes(xend = year,
                   y = coef - 1.96*se, yend = coef + 1.96*se), lwd = 1,
               colour = "grey") +
  geom_point(cex = 2) +
  geom_segment(aes(xend = year,
                   y = coef - 1.37*se, yend = coef + 1.37*se),
               lwd = 1.25,
               colour = "black") + #83% CIs
  scale_x_continuous(breaks = YEAR, labels = YEAR) +
  labs(y = "Coefficient Estimate",
       x = "Year") +
  geom_hline(aes(yintercept = 0), lty = 2, color = "grey") +
  theme_classic() +
  theme(axis.text=element_text(size=18),
        axis.title=element_text(size=20,face="bold"))
p2 <- ggplot(subset(cdat, year %in% YEAR), 
             aes(x = rr_sc)) +
  geom_bar(aes(y = ..prop..)) +
  facet_grid(~ year) +
  labs(y = "Proportion",
       x = "Racial Resentment") +
  theme_classic() +
  scale_x_continuous(labels = c(0, .25, .5, .75, 1))
grid.arrange(p1, p2, ncol = 1, heights = c(6, 4))
g <- arrangeGrob(p1, p2, ncol = 1, heights = c(7,3))
ggsave("./90_Tables and Figures/figure1.jpeg", g, dpi = 600, width = 10, height = 6.5)

# Printing coefficient for 2016 reported in text
outsub$coef[which(outsub$year == 2016)]
 
# Model to for hypothesis test comparing 2008 and 2016
m <- lm(rep_2pvote ~ rr_sc*as.factor(year) + pid7_sc*as.factor(year) + ideo_sc*as.factor(year) 
        + fem*as.factor(year) + age_sc*as.factor(year) + inc_sc*as.factor(year) + col*as.factor(year) + south*as.factor(year), 
        data = cdat, 
        subset = white == 1, 
        weights = weight)
f_test <- linearHypothesis(m, "rr_sc:as.factor(year)2008 = rr_sc:as.factor(year)2016", test = "F")
round(f_test$`Pr(>F)`, 3)

#---Figure 2a----
# Subsetting and stacking data
dat <- subset(anes, 
              white == 1 & web == 0 & year >= 1986,
              select = c("year", "rr_sc", "rep"))
dat16 <- subset(anes16,
                white == 1 & f2f == 1,
                select = c("rr_sc", "rep"))
dat16$year <- 2016
plot_dat <- rbind(dat, dat16)

# Generating means
d <- ddply(plot_dat, 
           .(year, rep), 
           summarise, 
           mean.rr = mean(rr_sc, na.rm = T))

d <- subset(d, !is.na(rep))
d$Party <- factor(d$rep, labels = c("Democrats", "Republicans"))

# Plotting
ggplot(d, aes(x = year, y = mean.rr, color = Party)) +
  geom_line(data = d[!is.na(d$mean.rr),], aes(lty = Party), size = 1) +
  geom_point(data = d[!is.na(d$mean.rr),], aes(shape = Party), size = 3) +
  scale_color_manual(values = c("black", "grey")) +
  scale_shape_manual(values = c(16, 15)) +
  labs(y = "Scale Mean",
       x = "Year") +
  theme_classic() +
  theme(legend.title = element_blank(),
        legend.position = "bottom",
        axis.text=element_text(size=14),
        axis.title=element_text(size=18,face="bold")) +
  scale_x_continuous(breaks = seq(1986, 2016, 6)) +
  scale_y_continuous(breaks = seq(0,1,.2), limits = c(0,1))
ggsave("./90_Tables and Figures/figure2a.jpeg", dpi = 600, width = 6, height = 4)

# Printing party differences reported in text
round(d$mean.rr[which(d$Party == "Republicans" & d$year == 1986)] - d$mean.rr[which(d$Party == "Democrats" & d$year == 1986)], 3)
round(d$mean.rr[which(d$Party == "Republicans" & d$year == 2016)] - d$mean.rr[which(d$Party == "Democrats" & d$year == 2016)], 3)

#---Figure 2b----
dat <- subset(anes, 
              white == 1 & web == 0 & year >= 1986,
              select = c("year", "rr_sc", "rep"))
dat16 <- subset(anes16,
                white == 1 & f2f == 1,
                select = c("rr_sc", "rep"))
dat16$year <- 2016
plot_dat <- rbind(dat, dat16)
## Generating Counts
counts88 <- prop.table(table(plot_dat$rep[which(plot_dat$year == 1988 & plot_dat$rep == 0
                                            | plot_dat$year == 1988 & plot_dat$rep == 1)],
                             plot_dat$rr_sc[which(plot_dat$year == 1988 & plot_dat$rep == 0
                                              | plot_dat$year == 1988 & plot_dat$rep == 1)]),1)
counts92 <- prop.table(table(plot_dat$rep[which(plot_dat$year == 1992 & plot_dat$rep == 0
                                            | plot_dat$year == 1992 & plot_dat$rep == 1)],
                             plot_dat$rr_sc[which(plot_dat$year == 1992 & plot_dat$rep == 0
                                              | plot_dat$year == 1992 & plot_dat$rep == 1)]),1)
counts04 <- prop.table(table(plot_dat$rep[which(plot_dat$year == 2004 & plot_dat$rep == 0
                                            | plot_dat$year == 2004 & plot_dat$rep == 1)],
                             plot_dat$rr_sc[which(plot_dat$year == 2004 & plot_dat$rep == 0
                                              | plot_dat$year == 2004 & plot_dat$rep == 1)]),1)
counts08 <- prop.table(table(plot_dat$rep[which(plot_dat$year == 2008 & plot_dat$rep == 0
                                            | plot_dat$year == 2008 & plot_dat$rep == 1)],
                             plot_dat$rr_sc[which(plot_dat$year == 2008 & plot_dat$rep == 0
                                              | plot_dat$year == 2008 & plot_dat$rep == 1)]),1)
counts12 <- prop.table(table(plot_dat$rep[which(plot_dat$year == 2012 & plot_dat$rep == 0
                                            | plot_dat$year == 2012 & plot_dat$rep == 1)],
                             plot_dat$rr_sc[which(plot_dat$year == 2012 & plot_dat$rep == 0
                                              | plot_dat$year == 2012 & plot_dat$rep == 1)]),1)
counts16 <- prop.table(table(plot_dat$rep[which(plot_dat$year == 2016 & plot_dat$rep == 0
                                            | plot_dat$year == 2016 & plot_dat$rep == 1)],
                             plot_dat$rr_sc[which(plot_dat$year == 2016 & plot_dat$rep == 0
                                              | plot_dat$year == 2016 & plot_dat$rep == 1)]),1)
# Plotting
jpeg("./90_Tables and Figures/figure2b.jpeg", width = 8, height = 6, units = "in", res = 600)
par(mfrow = c(2,3))
barplot(counts88, beside = T, col = c("black", "white"), 
        axes = F, ylim = c(0,.16),
        main = "1988", xlab = "", ylab = "Proportion")
legend("topleft", c("Democrats", "Republicans"), fill = c("black", "white"), bty = "n")
axis(2, las = 1)
barplot(counts92, beside = T, col = c("black", "white"), 
        axes = F, ylim = c(0,.16),
        main = "1992", xlab = "", ylab = "Proportion")
axis(2, las = 1)
barplot(counts04, beside = T, col = c("black", "white"), 
        axes = F, ylim = c(0,.16),
        main = "2004", xlab = "", ylab = "Proportion")
axis(2, las = 1)
barplot(counts08, beside = T, col = c("black", "white"), 
        axes = F, ylim = c(0,.16),
        main = "2008", xlab = "", ylab = "Proportion")
axis(2, las = 1)
barplot(counts12, beside = T, col = c("black", "white"), 
        axes = F, ylim = c(0,.16),
        main = "2012", xlab = "", ylab = "Proportion")
axis(2, las = 1)
barplot(counts16, beside = T, col = c("black", "white"), 
        axes = F, ylim = c(0,.16),
        main = "2016", xlab = "", ylab = "Proportion")
axis(2, las = 1)
dev.off()

# Chi-squared test reported in text
chisq_test <- summary(table(plot_dat$rep[which(plot_dat$year == 1988 & plot_dat$rep == 0
                             | plot_dat$year == 1988 & plot_dat$rep == 1)],
              plot_dat$rr_sc[which(plot_dat$year == 1988 & plot_dat$rep == 0
                               | plot_dat$year == 1988 & plot_dat$rep == 1)]))
round(chisq_test$statistic, 2)
round(chisq_test$p.value, 3)

# Modal respondent reported in text
which.max(prop.table(table(plot_dat$rr_sc[which(plot_dat$year == 1988 & plot_dat$rep == 0)])))
which.max(prop.table(table(plot_dat$rr_sc[which(plot_dat$year == 1988 & plot_dat$rep == 1)])))

#---Figure 3----
# adding variables 
anes16$weight <- anes16$weight.full
anes16$year <- 2016

# stacking data sets for analyses
cdat <- rbind(anes[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                      "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")], 
              anes16[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                        "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")])

cdat <- subset(cdat, year >= 1986)


YEAR <- unique(cdat$year[which(!is.na(cdat$rr_sc) & !is.na(cdat$rep_2pvote))])

# Looping to get coefficients and standard errors
for(i in 1:length(YEAR)){
  m <- lm(rep_2pvote ~ rr_sc, 
          data = cdat, 
          subset = white == 1 & year == YEAR[i] & rep == 0, 
          weights = weight)
  COEF <- coef(m)
  SE <- sqrt(diag(vcov(m)))[-1]
  x <- cbind(COEF, SE, rep(YEAR[i], length(COEF)))
  assign(paste("OUT_dem_bivar", YEAR[i], sep = "."), x)
  m <- lm(rep_2pvote ~ rr_sc, 
          data = cdat, 
          subset = white == 1 & year == YEAR[i] & rep == 1, 
          weights = weight)
  COEF <- coef(m)
  SE <- sqrt(diag(vcov(m)))[-1]
  x <- cbind(COEF, SE, rep(YEAR[i], length(COEF)))
  assign(paste("OUT_rep_bivar", YEAR[i], sep = "."), x)
}

# Collecting results by party into dataframe
DFs_dem <- grep("OUT_dem_bivar", ls(), value = T)
OUT.dem.bivar.year <- data.frame(coef = NA, se = NA, year = NA)
for(i in DFs_dem){
  x <- get(i)
  colnames(x) <- c("coef", "se", "year")
  OUT.dem.bivar.year <- rbind(OUT.dem.bivar.year, x)
}
OUT.dem.bivar.year <- OUT.dem.bivar.year[-1,]
OUT.dem.bivar.year$var <- rep(rownames(OUT_dem_bivar.1988), length(DFs_dem))
rownames(OUT.dem.bivar.year) <- NULL
OUT.dem.bivar.year$model <- "Democrats"

DFs_rep <- grep("OUT_rep_bivar", ls(), value = T)
OUT.rep.bivar.year <- data.frame(coef = NA, se = NA, year = NA)
for(i in DFs_rep){
  x <- get(i)
  colnames(x) <- c("coef", "se", "year")
  OUT.rep.bivar.year <- rbind(OUT.rep.bivar.year, x)
}
OUT.rep.bivar.year <- OUT.rep.bivar.year[-1,]
OUT.rep.bivar.year$var <- rep(rownames(OUT_rep_bivar.1988), length(DFs_rep))
rownames(OUT.rep.bivar.year) <- NULL
OUT.rep.bivar.year$model <- "Republicans"

# Stacking Results
OUT.party.bivar.year <- rbind(OUT.dem.bivar.year, OUT.rep.bivar.year)

OUT.party.bivar.year$var <- factor(OUT.party.bivar.year$var, 
                                   levels = c("rr_sc", "(Intercept)"),
                                   labels = c("Racial Resentment", "Intercept"))
OUT.party.bivar.year$year[which(OUT.party.bivar.year$model == "Democrats")] <- OUT.party.bivar.year$year[which(OUT.party.bivar.year$model == "Democrats")] - .25
OUT.party.bivar.year$year[which(OUT.party.bivar.year$model == "Republicans")] <- OUT.party.bivar.year$year[which(OUT.party.bivar.year$model == "Republicans")] + .25

# Plotting
ggplot(OUT.party.bivar.year, 
       aes(x = year, y = coef)) +
  geom_hline(aes(yintercept = 0), lty = 2, color = "grey") +
  geom_segment(aes(xend = year,
                   y = coef - 1.96*se, 
                   yend = coef + 1.96*se), lwd = 1,
               colour = "grey") +
  geom_segment(aes(xend = year,
                   y = coef - 1.37*se, yend = coef + 1.37*se),
               lwd = 1.25,
               colour = "black") + 
  geom_point(aes(shape = model), size = 3, stroke = 1) +
  scale_x_continuous(breaks = YEAR, labels = YEAR) +
  scale_shape_manual(name = "Party", values = c(0, 16)) +
  facet_grid(var~., scales = "free_y") +
  labs(y = "Coefficient Estimate",
       x = "Year") +
  theme_classic() +
  theme(legend.position = "bottom",
        legend.title=element_blank(),
        legend.text = element_text(size=16),
        strip.text = element_text(size=16),
        axis.text = element_text(size=18),
        axis.title = element_text(size=20, face="bold"))
ggsave("./90_Tables and Figures/figure3.png", width = 10, height = 6.5)

## Models to for hypothesis tests reported in text
# Republicans
rep_mod <- lm(rep_2pvote ~ rr_sc*as.factor(year), 
              data = cdat, 
              subset = white == 1 & rep == 1, 
              weights = weight)
summary(rep_mod)
# 2016 versus 1988
round(summary(rep_mod)$coefficients["rr_sc:as.factor(year)2016", "Pr(>|t|)"], 3)

# 2016 versus 1992
f_test92 <- linearHypothesis(rep_mod, "rr_sc:as.factor(year)1992 = rr_sc:as.factor(year)2016", test = "F")
round(f_test92$`Pr(>F)`, 3)

# Democrats
dem_mod <- lm(rep_2pvote ~ rr_sc*as.factor(year), 
              data = cdat, 
              subset = white == 1 & rep == 0, 
              weights = weight)
summary(dem_mod)

# 2016 versus 1988
round(summary(dem_mod)$coefficients["rr_sc:as.factor(year)2016", "Pr(>|t|)"], 3)
# 2016 versus 2008
f_test08 <- linearHypothesis(dem_mod, "rr_sc:as.factor(year)2008 = rr_sc:as.factor(year)2016", test = "F")
round(f_test08$`Pr(>F)`, 3)

#---Table 1----
# subsetting data frame
vsg.sub <- na.omit(subset(vsg.dat, white12 == 1,
                          select = c("case_identifier", "rep_2pvote.12","rep_2pvote.16",
                                     "rr_sc.12", "pid7_sc.12", "ideo5.r_12.sc", "fem", "age_12_sc",    
                                     "inc_12.sc", "col_12", "south_12", "weight")))
# Creating data frames to stack
vsg12 <- subset(vsg.sub,
                select = c("case_identifier", "rep_2pvote.12",
                           "rr_sc.12", "pid7_sc.12", "ideo5.r_12.sc", "fem", "age_12_sc",    
                           "inc_12.sc", "col_12", "south_12", "weight"))
vsg16 <- subset(vsg.sub,
                select = c("case_identifier", "rep_2pvote.16",
                           "rr_sc.12", "pid7_sc.12", "ideo5.r_12.sc", "fem", "age_12_sc",    
                           "inc_12.sc", "col_12", "south_12", "weight"))
# editing variable labels
vsg12$rep_2pvote <- vsg12$rep_2pvote.12
vsg12$rep_2pvote.12 <- NULL
vsg12$y16 <- 0

vsg16$rep_2pvote <- vsg16$rep_2pvote.16
vsg16$rep_2pvote.16 <- NULL
vsg16$y16 <- 1

# stacking data
vsg_stack <- rbind(vsg12, vsg16)

# estimating model
m1216 <- lm(rep_2pvote ~ rr_sc.12*y16 + pid7_sc.12*y16 + ideo5.r_12.sc*y16 +
              + fem*y16 + age_12_sc*y16 + inc_12.sc*y16 + col_12*y16 + south_12*y16, 
            data = vsg_stack, 
            weights = weight)
summary(m1216)

# Generate Results Table
stargazer(m1216,
          title = "Priming Whites' Racial Attitudes in 2016",
          model.numbers = F,
          covariate.labels = c("Racial Resentment",
                               "---*2016",
                               "Partisanship", 
                               "---*2016", 
                               "Ideology",
                               "---*2016",
                               "Female", 
                               "---*2016", 
                               "Age", 
                               "---*2016", 
                               "Income", 
                               "---*2016", 
                               "College Degree", 
                               "---*2016", 
                               "Southerner",
                               "---*2016",
                               "2016"),
          order = c(1, 10, 3, 11, 4, 12, 5, 13, 6, 14, 7, 15, 8, 16, 9, 17, 2, 18),
          dep.var.caption = "",
          dep.var.labels = c("Republican Presidential Vote"),
          no.space = T, 
          notes.append = F,
          notes = c("OLS regression results. Standard errors in parentheses. Outcome scaled 0-100. Covariates scaled 0-1 and are measured in December 2011.",
                    "Analyses use survey weights."), 
          notes.align = "l",
          out = c("./90_Tables and Figures/table1.tex"),  
          table.placement = "t",
          digits = 2, omit.stat = c("f", "adj.rsq"), align = T, df = F,
          label = "vsg_tab", header= F,
          star.cutoffs = NA
)

# Printing coefficient for interaction between RR and 2016 reported in text
round(summary(m1216)$coefficients["rr_sc.12:y16", "Estimate"], 1)

#---Table 2----
invar_dat <- subset(vsg.dat, white12 == 1, 
                    select = c("case_identifier", "rr.deserve.12", "rr.specfav.12", "rr.thard.12", "rr.pdisc.12",
                               "rr.deserve.16", "rr.specfav.16", "rr.thard.16", "rr.pdisc.16"))

# Desired Fit Measures
FIT <- c("chisq","df","cfi", "tli", "srmr", "rmsea", "rmsea.ci.lower", "rmsea.ci.upper")

# Defining configural invariance model
mod_config <- '
rr12 =~ rr.thard.12 + rr.specfav.12 + rr.deserve.12 + rr.pdisc.12
rr16 =~ rr.thard.16 + rr.specfav.16 + rr.deserve.16 + rr.pdisc.16

## error variances
# temporal
rr.thard.12 ~~ rr.thard.16
rr.specfav.12 ~~ rr.specfav.16
rr.deserve.12 ~~ rr.deserve.16
rr.pdisc.12 ~~ rr.pdisc.16

# reverse wording
rr.thard.12 ~~ rr.specfav.12 + rr.specfav.16
rr.thard.16 ~~ rr.specfav.16 + rr.specfav.12
'
fit_config <- cfa(mod_config, invar_dat, mimic = "Mplus")
summary(fit_config, standardized = T)
round(fitMeasures(fit_config)[FIT], 3)

# Defining metric invariance model; constrains loadings
mod_metric <- '
rr12 =~ v1*rr.thard.12 + v2*rr.specfav.12 + v3*rr.deserve.12 + v4*rr.pdisc.12
rr16 =~ v1*rr.thard.16 + v2*rr.specfav.16 + v3*rr.deserve.16 + v4*rr.pdisc.16

## error variances
# temporal
rr.thard.12 ~~ rr.thard.16
rr.specfav.12 ~~ rr.specfav.16
rr.deserve.12 ~~ rr.deserve.16
rr.pdisc.12 ~~ rr.pdisc.16

# reverse wording
rr.thard.12 ~~ rr.specfav.12 + rr.specfav.16
rr.thard.16 ~~ rr.specfav.16 + rr.specfav.12
'

fit_metric <- cfa(mod_metric, invar_dat, mimic = "Mplus")
summary(fit_metric, standardized = T)
round(fitMeasures(fit_metric)[FIT], 3)

# Combining results for output
FIT_INVAR <- c("chisq","df", "cfi", "srmr", "rmsea")
invar_fit <- data.frame(rbind(fitMeasures(fit_config)[FIT_INVAR],
                              fitMeasures(fit_metric)[FIT_INVAR]))
invar_fit$chi2_change <- NA
invar_fit$chi2_change_p <- NA
invar_fit$cfi_change <- NA
invar_fit$srmr_change <- NA
invar_fit$rmsea_change <- NA

invar_fit$chi2_change[2] <- invar_fit$chisq[2] - invar_fit$chisq[1]
invar_fit$chi2_change_p[2] <- max(pchisq(invar_fit$chisq[2] - invar_fit$chisq[1], 
                                         invar_fit$df[2] - invar_fit$df[1], 
                                         lower.tail = F), .001)
invar_fit$chi2_change_p[2] <- paste0("$<$", invar_fit$chi2_change_p[2])
invar_fit$cfi_change[2] <- invar_fit$cfi[2] - invar_fit$cfi[1]
invar_fit$srmr_change[2] <- invar_fit$srmr[2] - invar_fit$srmr[1]
invar_fit$rmsea_change[2] <- invar_fit$rmsea[2] - invar_fit$rmsea[1]

# removing DF
invar_table <- invar_fit[,-2]
# formatting to generate table
colnames(invar_table) <- c("$\\chi^2$", "CFI", "SRMR", "RMSEA", "$\\Delta\\chi^2$", "$\\Delta\\chi^2$ p-value", "$\\Delta$CFI", "$\\Delta$SRMR", "$\\Delta$RMSEA")
rownames(invar_table) <- c("Configural", "Metric")

tab <- xtable(invar_table, digits = 3, 
              caption = "Measurement Invariance of Racial Resentment, 2012-2016",
              align = c("l", "c", "c", "c", "c", "c", "c", "c", "c", "c"),
              label = "cfa_invar",
              # specifying significant digits
              display = c("s", "fg", "fg", "fg", "fg", "fg", "s", "fg", "fg", "fg"))
NOTE <- list()
NOTE$pos <- list()
NOTE$pos[[1]] <- c(nrow(invar_table))
NOTE$command <- c("\\bottomrule \\multicolumn{9}{l}{\\multirow{2}{\\textwidth}{Note: The configural model freely estimates item loadings using the \\textit{try harder} item to define the dimensions. The metric model constrains item loadings to equality between 2012 and 2016. Data from 2012-2016 VOTER Survey.}}")
print(tab, caption.placement = "top",
      sanitize.text.function = function(str) gsub("$\backslash$", "\\", str, fixed = TRUE),
      comment = F,
      add.to.row = NOTE,
      hline.after = c(-1, 0),
      file = "./90_Tables and Figures/table2.tex")

## Printing hypothesis testing reported in the text
# change in chi square
round(invar_fit$chisq[2] - invar_fit$chisq[1], 1)
# change in degrees of freedom
invar_fit$df[2] - invar_fit$df[1]
# p-value
round(pchisq(invar_fit$chisq[2] - invar_fit$chisq[1], 
             invar_fit$df[2] - invar_fit$df[1], 
             lower.tail = F), 3)

# change in CFI
invar_fit$cfi[2] - invar_fit$cfi[1]