#----------------------------------------------------------------------------------------------------#
# Filename: appendix_analyses.R
# File purpose: Conduct analyses reported in Appendix of Engelhardt "Trumped by Race: Explanations for Race's Influence on Whites' Votes in 2016"
#----------------------------------------------------------------------------------------------------#

# setwd()

#---Install Packages----
source("00_package_install.R")

#---Load Packages----
library(car) 
library(stargazer)
library(gridExtra)
library(ggplot2)
library(ggthemes)
library(lavaan)
library(semTools)
library(xtable)
library(plyr)

#---Loading cleaning files----

source("01_anes_cumulative_cleaning.R")
source("01_anes_2016_cleaning.R")
source("01_vsg_cleaning.R")

#---Table A1----
# adding variables 
anes16$weight <- anes16$weight.full
anes16$year <- 2016

# stacking data sets for analyses
cdat <- rbind(anes[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                      "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")], 
              anes16[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                        "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")])

cdat <- subset(cdat, year >= 1986)

m88 <- lm(rep_2pvote ~  rr_sc + pid7_sc + ideo_sc +
            + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == 1988, 
          weights = weight)
m92 <- lm(rep_2pvote ~  rr_sc + pid7_sc + ideo_sc +
            + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == 1992, 
          weights = weight)
m00 <- lm(rep_2pvote ~  rr_sc + pid7_sc + ideo_sc +
            + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == 2000, 
          weights = weight)
m04 <- lm(rep_2pvote ~  rr_sc + pid7_sc + ideo_sc +
            + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == 2004, 
          weights = weight)
m08 <- lm(rep_2pvote ~  rr_sc + pid7_sc + ideo_sc +
            + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == 2008, 
          weights = weight)
m12 <- lm(rep_2pvote ~  rr_sc + pid7_sc + ideo_sc +
            + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == 2012, 
          weights = weight)
m16 <- lm(rep_2pvote ~  rr_sc + pid7_sc + ideo_sc +
            + fem + age_sc + inc_sc + col + south, 
          data = cdat, 
          subset = white == 1 & year == 2016, 
          weights = weight)

# Code to produce table
stargazer(m88, m92, m00, m04, m08, m12, m16,
          title = "Predictors of Republican Vote Choice, 1988-2016",
          model.numbers = F,
          dep.var.caption = "",
          dep.var.labels = "",
          column.labels = c("1988", "1992", "2000", "2004", "2008", "2012", "2016"),
          covariate.labels = c("Racial Resentment", "Partisanship (Republican)", "Ideology (Conservative)",
                               "Female", "Age", "Income", "College Degree", "Southerner"),
          notes.append = F,
          no.space = T, notes = c("OLS regression results. Standard errors in parentheses. Outcome scaled 0-100. Covariates scaled 0-1.",
                                  "Analyses use survey weights.",
                                  "1996 omitted because racial resentment was not collected."), 
          notes.align = "l", intercept.bottom = T, 
          out = c("./90_Tables and Figures/tableA1.tex"),
          digits = 3, omit.stat = c("f", "adj.rsq"), align = T, df = F,
          header = F, 
          star.cutoffs = NA
)
#---Figure A1----
# adding variables 
anes16$weight <- anes16$weight.full
anes16$year <- 2016

# stacking data sets for analyses
cdat <- rbind(anes[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                      "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")], 
              anes16[,c("fem", "col", "age_sc", "inc_sc", "south", "rr_sc", "rep_2pvote", 
                        "year", "pid7_sc", "ideo_sc", "white", "weight", "rep")])

cdat <- subset(cdat, year >= 1986)

YEAR <- unique(cdat$year[which(!is.na(cdat$rr_sc) & !is.na(cdat$rep_2pvote))])
for(i in 1:length(YEAR)){
  m <- lm(rep_2pvote ~ rr_sc, 
          data = cdat, 
          subset = white == 1 & year == YEAR[i], 
          weights = weight)
  COEF <- coef(m)[-1]
  SE <- sqrt(diag(vcov(m)))[-1]
  x <- cbind(COEF, SE, rep(YEAR[i], length(COEF)))
  assign(paste("OUT_bivar", YEAR[i], sep = "."), x)
}

DFs <- grep("OUT_bivar.", ls(), value = T)
OUT.bivar.year <- data.frame(coef = NA, se = NA, year = NA)
for(i in DFs){
  x <- get(i)
  colnames(x) <- c("coef", "se", "year")
  OUT.bivar.year <- rbind(OUT.bivar.year, x)
}
OUT.bivar.year <- OUT.bivar.year[-1,]
OUT.bivar.year$var <- rep(rownames(OUT_bivar.1988), length(DFs))
rownames(OUT.bivar.year) <- NULL

OUT.bivar.year$var <- factor(OUT.bivar.year$var, levels = c("rr_sc"),
                             labels = c("Racial Resentment"))

# Generating Figure
p1 <- ggplot(OUT.bivar.year, 
             aes(x = year, y = coef)) +
  geom_segment(aes(xend = year,
                   y = coef - 1.96*se, yend = coef + 1.96*se), lwd = 1,
               colour = "grey") +
  geom_point(cex = 2) +
  geom_segment(aes(xend = year,
                   y = coef - 1.37*se, yend = coef + 1.37*se),
               lwd = 1.25,
               colour = "black") + #83% CIs
  scale_x_continuous(breaks = YEAR, labels = YEAR) +
  labs(y = "Coefficient Estimate",
       x = "Year") +
  geom_hline(aes(yintercept = 0), lty = 2, color = "grey") +
  theme_classic() +
  theme(axis.text=element_text(size=18),
        axis.title=element_text(size=20,face="bold"))
p2 <- ggplot(subset(cdat, year %in% YEAR), 
             aes(x = rr_sc)) +
  geom_bar(aes(y = ..prop..)) +
  facet_grid(~ year) +
  labs(y = "Proportion",
       x = "Racial Resentment") +
  theme_classic() +
  scale_x_continuous(labels = c(0, .25, .5, .75, 1))
grid.arrange(p1, p2, ncol = 1, heights = c(6, 4))
g <- arrangeGrob(p1, p2, ncol = 1, heights = c(7,3))
ggsave("./90_Tables and Figures/figureA1.jpeg", g, dpi = 600, width = 10, height = 6.5)

#---Table A2----
invar_dat <- subset(vsg.dat, white12 == 1, 
                    select = c("case_identifier", "rr.deserve.12", "rr.specfav.12", "rr.thard.12", "rr.pdisc.12",
                               "rr.deserve.16", "rr.specfav.16", "rr.thard.16", "rr.pdisc.16"))

# Desired Fit Measures
FIT <- c("chisq","df","cfi", "tli", "srmr", "rmsea", "rmsea.ci.lower", "rmsea.ci.upper")

# Defining configural invariance model
mod_config <- '
rr12 =~ rr.thard.12 + rr.specfav.12 + rr.deserve.12 + rr.pdisc.12
rr16 =~ rr.thard.16 + rr.specfav.16 + rr.deserve.16 + rr.pdisc.16

## error variances
# temporal
rr.thard.12 ~~ rr.thard.16
rr.specfav.12 ~~ rr.specfav.16
rr.deserve.12 ~~ rr.deserve.16
rr.pdisc.12 ~~ rr.pdisc.16

# reverse wording
rr.thard.12 ~~ rr.specfav.12 + rr.specfav.16
rr.thard.16 ~~ rr.specfav.16 + rr.specfav.12
'
fit_config <- cfa(mod_config, invar_dat, mimic = "Mplus")
summary(fit_config, standardized = T)
round(fitMeasures(fit_config)[FIT], 3)

# Defining metric invariance model; constrains loadings
mod_metric <- '
rr12 =~ v1*rr.thard.12 + v2*rr.specfav.12 + v3*rr.deserve.12 + v4*rr.pdisc.12
rr16 =~ v1*rr.thard.16 + v2*rr.specfav.16 + v3*rr.deserve.16 + v4*rr.pdisc.16

## error variances
# temporal
rr.thard.12 ~~ rr.thard.16
rr.specfav.12 ~~ rr.specfav.16
rr.deserve.12 ~~ rr.deserve.16
rr.pdisc.12 ~~ rr.pdisc.16

# reverse wording
rr.thard.12 ~~ rr.specfav.12 + rr.specfav.16
rr.thard.16 ~~ rr.specfav.16 + rr.specfav.12
'

fit_metric <- cfa(mod_metric, invar_dat, mimic = "Mplus")
summary(fit_metric, standardized = T)
round(fitMeasures(fit_metric)[FIT], 3)

# Full Results Table, rounding to 3 significant digits
items <- 15
OUT_INVAR <- array(NA, c(items, 4))
rownames(OUT_INVAR) <- c("Try Hard", "se th",
                         "Special Favors", "se sf",
                         "Deserve Less", "se dl",
                         "Past Discrimination", "se pd",
                         "$\\chi^2$", "DF", "CFI", "TLI", "SRMR", "RMSEA [90\\% CI]", "N")

OUT_INVAR <- as.data.frame(OUT_INVAR)

## Configural Model
est_config <- parameterEstimates(fit_config)
# 2012 
OUT_INVAR["Try Hard", 1] <- round(est_config$est[1], 3)
OUT_INVAR["se th", 1] <- paste0("(", round(est_config$se[1], 4), ")")
OUT_INVAR["Special Favors", 1] <- round(est_config$est[2], 3)
OUT_INVAR["se sf", 1] <- paste0("(", round(est_config$se[2], 4), ")")
OUT_INVAR["Deserve Less", 1] <- round(est_config$est[3], 3)
OUT_INVAR["se dl", 1] <- paste0("(", round(est_config$se[3], 4), ")")
OUT_INVAR["Past Discrimination", 1] <- round(est_config$est[4], 2)
OUT_INVAR["se pd", 1] <- paste0("(", round(est_config$se[4], 4), ")")

# 2016
OUT_INVAR["Try Hard", 2] <- round(est_config$est[5], 2)
OUT_INVAR["se th", 2] <- paste0("(", round(est_config$se[5], 4), ")")
OUT_INVAR["Special Favors", 2] <- round(est_config$est[6], 2)
OUT_INVAR["se sf", 2] <- paste0("(", round(est_config$se[6], 4), ")")
OUT_INVAR["Deserve Less", 2] <- round(est_config$est[7], 2)
OUT_INVAR["se dl", 2] <- paste0("(", round(est_config$se[7], 4), ")")
OUT_INVAR["Past Discrimination", 2] <- round(est_config$est[8], 2)
OUT_INVAR["se pd", 2] <- paste0("(", round(est_config$se[8], 4), ")")

OUT_INVAR["$\\chi^2$", 1] <- round(fitMeasures(fit_config)["chisq"])
OUT_INVAR["DF", 1] <- round(fitMeasures(fit_config)["df"], 3)
OUT_INVAR["CFI", 1] <- round(fitMeasures(fit_config)["cfi"], 3)
OUT_INVAR["TLI", 1] <- round(fitMeasures(fit_config)["tli"], 3)
OUT_INVAR["SRMR", 1] <- round(fitMeasures(fit_config)["srmr"], 4)
OUT_INVAR["RMSEA [90\\% CI]", 1] <- paste0(round(fitMeasures(fit_config)["rmsea"], 4),
                                           " [",
                                           round(fitMeasures(fit_config)["rmsea.ci.lower"], 4),
                                           ", ",
                                           round(fitMeasures(fit_config)["rmsea.ci.upper"], 4),
                                           "]")
OUT_INVAR["N", 1] <- nobs(fit_config)

## Metric Model
est_metric <- parameterEstimates(fit_metric)
# 2012 
OUT_INVAR["Try Hard", 3] <- round(est_metric$est[1], 2)
OUT_INVAR["se th", 3] <- paste0("(", round(est_metric$se[1], 4), ")")
OUT_INVAR["Special Favors", 3] <- round(est_metric$est[2], 2)
OUT_INVAR["se sf", 3] <- paste0("(", round(est_metric$se[2], 5), ")")
OUT_INVAR["Deserve Less", 3] <- round(est_metric$est[3], 2)
OUT_INVAR["se dl", 3] <- paste0("(", round(est_metric$se[3], 4), ")")
OUT_INVAR["Past Discrimination", 3] <- round(est_metric$est[4], 2)
OUT_INVAR["se pd", 3] <- paste0("(", round(est_metric$se[4], 4), ")")

# 2016
OUT_INVAR["Try Hard", 4] <- round(est_metric$est[5], 2)
OUT_INVAR["se th", 4] <- paste0("(", round(est_metric$se[5], 4), ")")
OUT_INVAR["Special Favors", 4] <- round(est_metric$est[6], 2)
OUT_INVAR["se sf", 4] <- paste0("(", round(est_metric$se[6], 5), ")")
OUT_INVAR["Deserve Less", 4] <- round(est_metric$est[7], 2)
OUT_INVAR["se dl", 4] <- paste0("(", round(est_metric$se[7], 4), ")")
OUT_INVAR["Past Discrimination", 4] <- round(est_metric$est[8], 2)
OUT_INVAR["se pd", 4] <- paste0("(", round(est_metric$se[8], 4), ")")

OUT_INVAR["$\\chi^2$", 3] <- round(fitMeasures(fit_metric)["chisq"])
OUT_INVAR["DF", 3] <- round(fitMeasures(fit_metric)["df"], 3)
OUT_INVAR["CFI", 3] <- round(fitMeasures(fit_metric)["cfi"], 3)
OUT_INVAR["TLI", 3] <- round(fitMeasures(fit_metric)["tli"], 3)
OUT_INVAR["SRMR", 3] <- round(fitMeasures(fit_metric)["srmr"], 4)
OUT_INVAR["RMSEA [90\\% CI]", 3] <- paste0(round(fitMeasures(fit_metric)["rmsea"], 4),
                                           " [",
                                           round(fitMeasures(fit_metric)["rmsea.ci.lower"], 4),
                                           ", ",
                                           round(fitMeasures(fit_metric)["rmsea.ci.upper"], 4),
                                           "]")
OUT_INVAR["N", 3] <- nobs(fit_metric)

colnames(OUT_INVAR) <- c("2011", "2016", "2011", "2016")

tab <- xtable(OUT_INVAR, 
              label = c("invar_full"), 
              align = c("l", rep("c", ncol(OUT_INVAR))),
              caption = c("Measurement Invariance Results"))
NOTE <- list()
NOTE$pos <- list()
NOTE$pos[[1]] <- c(nrow(OUT_INVAR))
NOTE$command <- c("\\bottomrule \\multicolumn{5}{l}{\\multirow{4}{\\textwidth}{Note: Models estimated using maximum likelihood. Parameter estimates with standard errors in parentheses. Estimated error-variances are omitted.}}")
print(tab,
      caption.placement = "top",
      sanitize.text.function = function(str) gsub("$\backslash$", "\\", str, fixed = TRUE),
      comment = F,
      add.to.row = NOTE,
      hline.after = c(-1, 0),
      file = "./90_Tables and Figures/tableA2.tex")